package rpc

import (
	"context"
	"encoding/json"

	"git.loafle.net/commons_go/logging"
	"git.loafle.net/commons_go/rpc"
	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol"
	crpj "git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newGatewayRPCServletHandler() GatewayRPCServletHandler {
	sh := &GatewayRPCServletHandlers{}
	sh.RegisterCodec(crpj.Name, crpj.NewServerCodec())
	return sh
}

type GatewayRPCServletHandlers struct {
	gateway.ServletHandlers
}

func (sh *GatewayRPCServletHandlers) Invoke(servletCTX rpc.ServletContext, requestCodec protocol.RegistryCodec) (result interface{}, err error) {
	defer func() {
		if nil != err {
			logging.Logger().Errorf("Gateway Websocket: Inovke error %v", err)
		}

		if !requestCodec.HasResponse() {
			result = nil
			err = nil
		}
	}()

	md := servletCTX.GetAttribute(oogw.GRPCMetadataKey)
	if nil == md {
		md = metadata.Pairs(
			oogw.GRPCClientTypeKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string),
			oogw.GRPCSessionIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string),
			oogw.GRPCTargetIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string))

		servletCTX.SetAttribute(oogw.GRPCMetadataKey, md)
	}

	grpcCTX := metadata.NewOutgoingContext(context.Background(), md.(metadata.MD))

	var (
		grpcResult string
		params     []string
	)

	params, err = requestCodec.Params()
	if nil != err {
		return
	}

	grpcResult, err = grpc.Exec(grpcCTX, requestCodec.Method(), params)
	if err != nil {
		return
	}

	if !requestCodec.HasResponse() {
		return
	}

	err = json.Unmarshal([]byte(grpcResult), &result)
	if err != nil {
		return
	}

	return
}

func (sh *GatewayRPCServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

}
